div.recurrence-widget {
    float: left;
    border: 1px solid #ccc;
}

div.recurrence-widget div.panel {
    border-bottom: 1px solid #ddd;
}

div.recurrence-widget select,
div.recurrence-widget input {
    margin: 0 .5em;
}

div.recurrence-widget a.recurrence-label {
    display: block;
    cursor: pointer;
    padding: .5em .75em;
    font-weight: bold;
    white-space: nowrap;
    padding-right: 3em;
    color: black;
}

div.recurrence-widget .remove,
div.recurrence-widget .remove:link,
div.recurrence-widget .remove:visited {
    float: left;
    padding: .25em .5em;
    font-size: 1.1em;
    font-weight: bold;
    cursor: pointer;
    text-decoration: none;
    color: #555;
    line-height: 1.4em;
}

div.recurrence-widget .remove:hover {
    color: #a00;
}

div.recurrence-widget .control {
    padding: .5em 1em .5em .5em;
}

div.recurrence-widget .add-button,
div.recurrence-widget .add-button:link,
div.recurrence-widget .add-button:visited {
    color: green;
}

div.recurrence-widget .plus {
    font-size: 1.2em;
    font-weight: bold;
    padding-right: .5em;
}

div.recurrence-widget .add-button:hover {
    color: green;
}

div.recurrence-widget .control .add-date {
    padding-left: 1em;
    margin-left: 1em;
    border-left: 1px solid #ddd;
}

div.recurrence-widget .freq {
    display: inline
}

div.recurrence-widget .interval {
    display: inline;
    padding-left: 1em;
}

div.recurrence-widget form {
    overflow: hidden;
    padding: .5em 2.2em;
    border-top: 1px solid #ddd;
}

div.recurrence-widget .form {
    padding: .5em 0;
}

div.recurrence-widget .radio {
    margin-left: 0;
}

div.recurrence-widget .form ul {
    margin: 0;
    padding: 0;
}

div.recurrence-widget .form ul li {
    list-style-type: none;
}

div.recurrence-widget .section {
    margin: .5em 1em;
}

div.recurrence-widget .until-count {
    margin: 0;
    padding: 0;
}

div.recurrence-widget .until-count li {
    margin: 0;
    padding: 0;
    display: inline;
    list-style-type: none;
}

div.recurrence-widget .until-count .count {
    padding-left: 1em;
}

div.recurrence-widget .monthday .grid {
    margin-left: .5em;
}

div.recurrence-widget .grid {
    border: 0;
    border-top: 1px solid #ccc;
    border-left: 1px solid #ccc;
}

div.recurrence-widget .grid td {
    text-align: center;
    border-bottom: 1px solid #ccc;
    border-right: 1px solid #ccc;
    padding: .8em;
    cursor: pointer;
}

div.recurrence-widget .grid td.active {
    background-color: #ddd;
}

div.recurrence-widget .grid td.empty {
    border: 0;
}

div.recurrence-widget .grid-disabled td.active {
    background-color: #ddd;
}

div.recurrence-widget .disabled {
    opacity: .75;
    filter: alpha(opacity=75);
}

div.recurrence-widget .limit ul {
    margin-top: .5em;
}

div.recurrence-widget .weekly .section {
    margin: 1em 0;
}

div.recurrence-widget .weekly td {
    width: 2.5em;
}

div.recurrence-widget .yearly table {
    margin: .5em 0 1em 0;
}

div.recurrence-widget .yearly td {
    padding: 1em 1.5em;
}

div.recurrence-widget .checkbox {
    margin-left: 0;
}

div.recurrence-widget .until-count {
    margin-left: 1.5em;
}

div.recurrence-widget .mode {
    padding: .25em 0 1em 0;
}

div.recurrence-widget .date-selector .calendar-button {
    background: url('../img/recurrence-calendar-icon.png') no-repeat center;
    padding: .25em;
}

table.recurrence-calendar {
    position: absolute;
    background-color: white;
    border-top: 1px solid #ccc;
    border-left: 1px solid #ccc;
    /*outline: 3px solid white;*/
    margin: 0;
}

table.recurrence-calendar td {
    padding: 0;
}

table.recurrence-calendar a,
table.recurrence-calendar a:link,
table.recurrence-calendar a:visited {
    color: #555;
    font-weight: bold;
}

table.recurrence-calendar .year,
table.recurrence-calendar .navigation {
    border-right: 1px solid #ccc;
}

table.recurrence-calendar .body td {
    border: 0;
    border-bottom: 1px solid #ccc;
    border-right: 1px solid #ccc;
}

table.recurrence-calendar .body td.header {
    border-top: 1px solid #ccc;
    font-weight: bold;
    color: #555;
}

table.recurrence-calendar .body td.header,
table.recurrence-calendar .body td.day {
    padding: .25em;
    text-align: center;
    vertical-align: center;
}

table.recurrence-calendar .body td.day {
    cursor: pointer;
}

table.recurrence-calendar .body td.active {
    background-color: #ddd;
}

table.recurrence-calendar .body td.empty {
    background-color: #eee;
}

table.recurrence-calendar .year {
    padding-top: .25em;
    padding-right: 2em;
    text-align: center;
}

table.recurrence-calendar .navigation {
    width: 100%;
}

table.recurrence-calendar .navigation {
    text-align: center;
}

table.recurrence-calendar .navigation td.prev-year,
table.recurrence-calendar .navigation td.next-year,
table.recurrence-calendar .navigation td.prev-month,
table.recurrence-calendar .navigation td.next-month {
    /*width: 1%;*/
    font-size: .8em;
}

table.recurrence-calendar .remove,
table.recurrence-calendar .remove:link,
table.recurrence-calendar .remove:visited {
    float: left;
    padding: .1em .25em;
    padding-bottom: 0;
    font-size: 1.1em;
    font-weight: bold;
    cursor: pointer;
    text-decoration: none;
    color: #555;
    line-height: 1.2em;
}

table.recurrence-calendar .remove:hover {
    color: #a00;
}

table.recurrence-calendar .footer {
    height: .5em;
    overflow: hidden;
    border-bottom: 1px solid #ccc;
    border-right: 1px solid #ccc;
}

